
#include "interface.h"

#include "suborLevel.h"
#include "zbrane.h"
#include "bulanec.h"
#include "panel.h"
#include "strela.h"
#include "prekazka.h"
#include "vec.h"
#include "animacia.h"
#include "robot.h"
#include "teleport.h"
#include "trubky.h"

#include "kurzor.h"
#include "mainMenu.h"
#include "vyberLevel.h"
#include "hraSa.h"
#include "nastavenia.h"
#include "tabulka.h"
#include "autory.h"

#include "widget_button.h"
#include "widget_check.h"
#include "widget_textfield.h"

#include "jazyk.h"
#include "zvuky.h"
#include "hudba.h"

#ifdef NET_MULTIPLAYER
#include "siet.h"
#endif

//povrch okna
SDL_Surface *screen;

//zoznam ramcov, ktore sa maju prekreslit
prekresli *zoznamPrekresli;


myimage *zoznamImage;

//font pisma
TTF_Font *g_font;

//casovac
static SDL_TimerID timer;

//farba pisma
static SDL_Color farba_pisma = {255,255,255,0};// Barva textu

//priznaky okna
static Uint32 g_win_flags = SDL_HWSURFACE|SDL_DOUBLEBUF|SDL_ANYFORMAT;

//koniec programu?
static int isExit = 0;

static int miesto_v_hre;

//sprava ktora sa zobrazuje na obrazovke
static char sprava[STR_SIZE];

#ifdef NET_MULTIPLAYER

//zamok na uzamknutie obrazovky
SDL_mutex *mutex_display;

//som SERVER alebo CLIENT
int stav;

#endif

int mycas;
//#define BECHMARK_AKCIA

/* *************************************************************************** */

int init()
{
	//Nacteni a nastaveni jazyku
	initJazyk();
	
	//inicializujem SDL
	if(SDL_Init(SDL_SUBSYSTEMS) == -1)
	{
		fprintf(stderr, "%s: %s\n",Text(TEXT_SDL_ERROR_1),SDL_GetError());
		SDL_Quit();
		return 1;
	}

	//inicializujem okno
	screen = SDL_SetVideoMode(WINDOW_SIZE_X,WINDOW_SIZE_Y,0,g_win_flags);

	if(screen == NULL)
	{	
		fprintf(stderr, "%s: %s\n",Text(TEXT_SDL_ERROR_2),SDL_GetError());
		SDL_Quit();
		return 2;
	}

	//nahravam ikonku
	SDL_Surface *ikonka = loadImage(PATH_IMAGE "ikonka.png",1);
	if(ikonka == NULL)
	{
		fprintf(stderr, "%s: %s\n",Text(TEXT_SDL_ERROR_3),SDL_GetError());
    	}
    	else
	{
        	SDL_WM_SetIcon(ikonka, NULL);
        	SDL_FreeSurface(ikonka);
    	}

	//inicializujem font
	if(TTF_Init() == -1)
	{
		fprintf(stderr, "%s: %s\n",Text(TEXT_SDL_ERROR_4),SDL_GetError());
		return 0;
	}
	
	//nahravam font
	g_font = TTF_OpenFont(PATH_FONTS "DejaVuSans.ttf",16);

	SDL_WM_SetCaption(WINDOW_TITLE, NULL);
	SDL_ShowCursor(0);
	SDL_EnableKeyRepeat(1,1);
	//miesto_v_hre=HRA_SA;
	timer = SDL_AddTimer(INTERVAL,TimerCallback,NULL);
	zoznamPrekresli=NULL;
	zoznamImage=NULL;
	odstranSpravu();
	srand((unsigned)time(NULL));

	return 0;
}

void vytvorDomovskyAdresar()
{
	char home[STR_SIZE];
	char *homeDirector;

#ifdef WIN32
	homeDirector=getenv("USERPROFILE");
#else
	homeDirector=getenv("HOME");
#endif

	if(homeDirector==NULL)
	{
		fprintf(stderr,"Neni domovsky adresar pro hru!\n");
		exit(0);
	}

#ifdef WIN32
	sprintf(home, "%s/%s", homeDirector, "Tuxanci");
#else
	sprintf(home, "%s/%s", homeDirector, ".tuxanci");
#endif


	if (access(home, F_OK)!=0)
	{
		printf("Vytvarim adresar %s\n", home);
#ifndef WIN32
		if (mkdir(home, 0755)!=0) {
#else
		if (mkdir(home)!=0) {
#endif
			fprintf(stderr, "Can't create home directory!\n");
			exit(0);
		}
	}

	mycas=0;
}
/* *************************************************************************** */


void pridajImage(myimage **p,SDL_Surface *img,
		int x,int y,int w,int h,
		int px,int py,int pw,int ph,
		int vrstva,int pflagredraw)
{
	myimage *new;
	myimage *teraz;
	myimage *prev;

	new = malloc(sizeof(myimage));
	new->x=x;
	new->y=y;
	new->w=w;
	new->h=h;
	new->px=px;
	new->py=py;
	new->pw=pw;
	new->ph=ph;
	new->vrstva=vrstva;
	new->image=img;
	new->flagredraw=pflagredraw;
	teraz=*p;
	prev=NULL;

	//vyhladaj svoju vrstvu
	while(teraz!=NULL && teraz->vrstva<new->vrstva)
	{
		prev=teraz;
		teraz=teraz->next;
	}

	//porovnaj s vyskami na tvojej vrstve,
	//ak neexistuje, rovno to tam hod
	while(teraz!=NULL && teraz->y+teraz->h<new->y+new->h && teraz->vrstva==new->vrstva)
	{
		prev=teraz;
		teraz=teraz->next;
	}

	new->next=teraz;

	if(prev!=NULL)
		prev->next=new;
	else
		(*p)=new;
}

/*
 * Vykresli zoznam *p na obrazovku
 */
void zoznamImageShow(myimage **p)
{
	myimage *teraz;
	myimage *del;
	SDL_Rect dst_rect,src_rect;

	teraz=*p;

	while(teraz!=NULL)
	{
		dst_rect.x=teraz->x;
		dst_rect.y=teraz->y;
	
		src_rect.x=teraz->px;
		src_rect.y=teraz->py;
		src_rect.w=teraz->pw;
		src_rect.h=teraz->ph;
		
		SDL_BlitSurface(teraz->image,&src_rect,screen,&dst_rect);
		
		if(teraz->flagredraw)
		{
			pridajPrekresli(&zoznamPrekresli,teraz->x-10,teraz->y-10,teraz->w+20,teraz->h+20);
		}

		del=teraz;
		teraz=teraz->next;
		free(del);
	}

	*p=NULL;
}

/*
 *  Tady se sice pracuje i s bulankem, ale NESMI tu byt mutexy!!! 
 * */
void pridajPrekresli(prekresli **p,int x,int y,int w,int h)
{
	prekresli *teraz;

	teraz=malloc(sizeof(prekresli));
	teraz->x=x;
	teraz->y=y;
	teraz->w=w;
	teraz->h=h;

	if(teraz->x<=0)teraz->x=0;
	if(teraz->x>WINDOW_SIZE_X-1)teraz->x=WINDOW_SIZE_X-1;
	if(teraz->y<=0)teraz->y=0;
	if(teraz->y>WINDOW_SIZE_Y-1)teraz->x=WINDOW_SIZE_Y-1;
	if(teraz->x+teraz->w>=WINDOW_SIZE_X-1)teraz->w=WINDOW_SIZE_X-(teraz->x+1);
	if(teraz->y+teraz->h>=WINDOW_SIZE_Y-1)teraz->h=WINDOW_SIZE_Y-(teraz->y+1);
	if(teraz->x>=WINDOW_SIZE_X-1)teraz->x=-1;
	if(teraz->y>=WINDOW_SIZE_Y-1)teraz->y=-1;

	teraz->next=*p;
	*p=teraz;
}

void akciaPrekresli(prekresli **p)
{
	prekresli *teraz;
	prekresli *del;
	SDL_Rect dst_rect;
	//int prekresli=0;
	teraz=*p;
/*	
	mycas++;
	if(mycas==2)
	{
		prekresli=1;
		mycas=0;
	}
*/
	while(teraz!=NULL)
	{
		//printf("%d %d %d %d\n",teraz->x,teraz->y,teraz->w,teraz->h);
		//if(teraz->x!=-1 && teraz->y!=-1)SDL_UpdateRect(screen,teraz->x,teraz->y,teraz->w,teraz->h);
	
		dst_rect.x=teraz->x;
		dst_rect.y=teraz->y;
		dst_rect.w=teraz->w;
		dst_rect.h=teraz->h;
		if(teraz->x!=-1 && teraz->y!=-1)SDL_UpdateRects(screen,1,&dst_rect);

		del=teraz;
		teraz=teraz->next;
		free(del);
	}

	*p=NULL;
}

/* *************************************************************************** */

void setMiesto_v_hre(int x)
{
	miesto_v_hre=x;
	
	if(miesto_v_hre==MAIN_MENU || miesto_v_hre==TABULKA)
	{
		prehrajHudbu(HUDBA_HLAVNE_MENU);
	}

	if(miesto_v_hre==AUTORY)nastavTitulky();

	if(miesto_v_hre==KONIEC)
	{
		vsetkoZabal();
		exit(0);
	}

	kresli();
	SDL_Flip(screen);
}

/* *************************************************************************** */

Uint32 TimerCallback(Uint32 interval, void* param)
{
	SDL_Event event;
	event.type = SDL_USEREVENT;
	event.user.code = USR_EVT_TIMER;
	event.user.data1 = NULL;
	event.user.data2 = NULL;

	SDL_PushEvent(&event);

	return interval;
}

SDL_Surface *loadImage(const char *filename,int alpha)
{
	SDL_Surface *tmp;
	SDL_Surface *ret;
	
	if((tmp = IMG_Load(filename)) == NULL)
	{
		fprintf(stderr, "%s\n", SDL_GetError());
		return NULL;
	}

	if((ret = (alpha) ? SDL_DisplayFormatAlpha(tmp):SDL_DisplayFormat(tmp)) == NULL)
	{
		fprintf(stderr, "%s\n", SDL_GetError());
		SDL_FreeSurface(tmp);
		return NULL;
	}

	SDL_FreeSurface(tmp);
	return ret;
}

int toggleFullscreen()
{
	if(g_win_flags & SDL_FULLSCREEN)
		g_win_flags &= ~SDL_FULLSCREEN;
	else
		g_win_flags |= SDL_FULLSCREEN;

	if(SDL_WM_ToggleFullScreen(screen) == 0)
	{
		fprintf(stderr, "Nemozem prepnut do fullscreenu!\n");

		SDL_FreeSurface(screen);
		screen = SDL_SetVideoMode(WINDOW_SIZE_X,WINDOW_SIZE_Y,WIN_BPP, g_win_flags);

		if(screen == NULL)
		{
			fprintf(stderr, "Nemozem premnut do okna:%s\n",SDL_GetError());
			return 0;
		}

		kresli();
	}

	return 1;
}
/* *************************************************************************** */

void zobrazSpravu(char *s)
{
	strcpy(sprava,s);
	if(strlen(s)>0) printf("Message: %s\n",s);
}

void odstranSpravu()
{
	zobrazSpravu("");
}

void kresliSpravu()
{
	SDL_Rect dst_rect;
	SDL_Surface *text;
	int width,vyska;

	TTF_SizeText(g_font,sprava,&width,&vyska);
	text = TTF_RenderUTF8_Blended(g_font,sprava,farba_pisma);
	dst_rect.x=(WINDOW_SIZE_X-width)/2;
	dst_rect.y=WINDOW_SIZE_Y/2;
	SDL_BlitSurface(text,NULL,screen,&dst_rect);
	SDL_FreeSurface(text);
	pridajPrekresli(&zoznamPrekresli,100,dst_rect.y-10,WINDOW_SIZE_X-200,vyska+20);
}

int getSirkaTextu(const char *s)
{
	char *znak1[15] = {"š","č","ť","ž","ý","á","í","é","ň","ú","ä","ô","ó","ň","Ď"};
	char *znak2[15] = {"s","c","t","z","y","a","i","e","n","u","a","o","o","n","D"};

	char *str;
	char *p;
	int len;
	int ret=0;
	int i;

	len=strlen(s);

	str=malloc(len+1);
	strcpy(str,s);

	for(i=0;i<15;i++)
	{
		while((p=strstr(str,znak1[i]))!=NULL)
		{
			strncpy(p,znak2[i],strlen(znak2[i]));
		}
	}

	TTF_SizeText(g_font,s,&ret,NULL);
	//printf("pred=%d\n",ret);
	//TTF_SizeText(g_font,str,&ret,NULL);
	//printf("po=%d %s\n",ret,str);

	free(str);

	return ret;
}

void kresli()
{
	switch(miesto_v_hre)
	{
		case MAIN_MENU :
			kresliMainMenu();
		break;
		case HRA_SA :
			kresliHraSa();
		break;
		case VYBER_LEVEL :
			kresliVyberLevel();
		break;
		case NASTAVENIA :
			kresliNastavenia();
		break;
		case TABULKA :
			kresliTabulka();
		break;
		case AUTORY :
			kresliAutory();
		break;
	}

	kresliSpravu();
	akciaPrekresli(&zoznamPrekresli);
}

/* *************************************************************************** */


int akcia()
{
	switch(miesto_v_hre)
	{
		case MAIN_MENU :
			akciaMainMenu();
		break;
		case HRA_SA :
			akciaHraSa();
		break;
		case VYBER_LEVEL :
			akciaVyberLevel();
		break;
		case NASTAVENIA :
			akciaNastavenia();
		break;
		case TABULKA :
			akciaTabulka();
		break;
		case AUTORY :
			akciaAutory();
		break;
	}

	kresli();

	return 0;
}

int eventAction()
{
	SDL_Event event;

	while(SDL_WaitEvent(&event))
	{
		switch(event.type)
		{
		case SDL_KEYDOWN:
			switch(event.key.keysym.sym)
			{
				case SDLK_ESCAPE:
					//Pokud nebudu v hlavnim menu, rad bych se do nej navratil,...
					//to se bohuzel nestane, ale to me zatim netrapi {Krakonos}
					if (miesto_v_hre==MAIN_MENU) 
					{
#ifdef NET_MULTIPLAYER
						sendEvent("quit",NULL);
#endif
						//setMiesto_v_hre(KONIEC);
					}

					if (miesto_v_hre==HRA_SA) 
					{
#ifdef NET_MULTIPLAYER
						sendEvent("quit",NULL);
#endif
						quitLevel();
					}
					if (miesto_v_hre==VYBER_LEVEL) 

					{
#ifdef NET_MULTIPLAYER
						sendEvent("quit",NULL);
#endif
						setMiesto_v_hre(MAIN_MENU);
					}
				break;

				case SDLK_F1:
					if(!toggleFullscreen())return 0;
				break;

				default:
				break;
			}
		break;

		// Uzivatelske udalosti (timer)
		case SDL_USEREVENT:
			switch(event.user.code)
			{
				case USR_EVT_TIMER:
					akcia();
				break;

				default:
				break;
			}
		break;

		// Zmena velikosti okna
		case SDL_VIDEORESIZE:
			screen = SDL_SetVideoMode(event.resize.w,event.resize.h, WIN_BPP, g_win_flags);

			if(screen == NULL)
			{
				fprintf(stderr, "Nemozem zmenit rozlisene okna: %s\n",SDL_GetError());
				return 0;
			}
		break;

		// Pozadavek na ukonceni
		case SDL_QUIT:
            // Tento kod jsem pridal, aby slo okno zavrit take krizkem (Paces)
#ifdef NET_MULTIPLAYER
			sendEvent("quit",NULL);
#endif
			setMiesto_v_hre(KONIEC);
		break;

		default:
		break;
		}
	}

	return 0;
}

void vsetkoZabal()
{
	TTF_Quit();
	SDL_Quit();

	quitSuborLevel();
	quitBulanec();
	quitPanel();
	quitKurzor();
	quitMainMenu();
	quitVyberLevel();
	quitHraSa();
	quitNastavenia();
	quitTabulka();
	quitAutory();

	quitButton();
	quitCheck();
	quitTextfield();

	quitAudio();
	quitHudba();
}

/* *************************************************************************** */

int main(int argc, char *argv[])
{
	vytvorDomovskyAdresar();

	init();

	initSuborLevelov();
	initBulanec();
	initPanel();
	initKurzor();
	initMainMenu();
	initVyberLevel();
	initHraSa();
	initNastavenia();
	initTabulka();
	initAutory();

	initButton();
	initCheck();
	initTextfield();

	initAudio();
	initHudba();

	zmenNastavenia();
	setMiesto_v_hre(MAIN_MENU);

#ifdef NET_MULTIPLAYER
	mutex_display=SDL_CreateMutex();
#endif

#ifdef NET_MULTIPLAYER
	if(argc>=2 && strcmp(argv[1],"-server")==0)
	{
		initServer();
		stav=SERVER;
	}

	if(argc>=2 && strcmp(argv[1],"-client")==0)
	{
		stav=CLIENT;
		initClient();
	}
#endif

	while(1)//!isExit)
	{
		isExit = !eventAction();
	}

	return 0;
}

/* *************************************************************************** */

