
#include "interface.h"
#include "zvuky.h"

#ifndef NO_SOUND

// zvuk pro mrtveho bulanka
static Mix_Chunk *z_smrt;

//vystreli
static Mix_Chunk *z_zvuk1;
static Mix_Chunk *z_zvuk2;
static Mix_Chunk *z_zvuk3;
static Mix_Chunk *z_mina;

//zvuk pri zobrani zbrane
static Mix_Chunk *z_zbran;
static Mix_Chunk *z_bonus;
static Mix_Chunk *z_teleport;
static Mix_Chunk *z_nabyjaSa;
static Mix_Chunk *z_nabyjaSa2;
static Mix_Chunk *z_opening;
static Mix_Chunk *z_odraz_zbran;
static Mix_Chunk *z_lasser;
static Mix_Chunk *z_odraz_r;
static Mix_Chunk *z_odraz_l;

static int zapnuta;


Mix_Chunk *nahrajZvuk(const char *subor)
{
	Mix_Chunk *zvuk;

	zvuk = Mix_LoadWAV(subor);
	
	if (!zvuk)
	{
		fprintf(stderr, "Nelze nahrat zvuk %s : %s\n", subor, Mix_GetError());
		return NULL;
	}

	return zvuk;
}

#endif

void initAudio()
{
#ifndef NO_SOUND
	if (SDL_Init(SDL_INIT_AUDIO)==-1)
	{
		fprintf(stderr, "Nelze inicializovat audio : %s\n", SDL_GetError());
		return;
	}
    
	if (Mix_OpenAudio(44100, MIX_DEFAULT_FORMAT, 2, 1024)==-1)
	{
        	fprintf(stderr, "Nelze inicializovat audio : %s\n", Mix_GetError());
        	return;
	}

	//jeste inicializovat kanaly a volume
	Mix_AllocateChannels(16);
	Mix_Volume(-1, MIX_MAX_VOLUME);

	//nahraj zvuky vystrelov
	z_zvuk1 = nahrajZvuk(PATH_SOUND "zvuk1.ogg");
	z_zvuk2 = nahrajZvuk(PATH_SOUND "zvuk2.ogg");
	z_zvuk3 = nahrajZvuk(PATH_SOUND "zvuk3.ogg");
	z_mina = nahrajZvuk(PATH_SOUND "mina.ogg");

	// nahrat zvuk mrtveho bulanka
	z_smrt = nahrajZvuk(PATH_SOUND "smrt.ogg");
	z_zbran = nahrajZvuk(PATH_SOUND "zbran.ogg");
	z_bonus = nahrajZvuk(PATH_SOUND "bonus.ogg");
	z_teleport = nahrajZvuk(PATH_SOUND "teleport.ogg");

	z_nabyjaSa = nahrajZvuk(PATH_SOUND "gun.ogg");
	z_nabyjaSa2 = nahrajZvuk(PATH_SOUND "gun2.ogg");

	z_opening = nahrajZvuk(PATH_SOUND "opening.ogg");

	z_lasser = nahrajZvuk(PATH_SOUND "laser.ogg");
	z_odraz_zbran = nahrajZvuk(PATH_SOUND "odraz-gun.ogg");
	z_odraz_r = nahrajZvuk(PATH_SOUND "odraz-r.ogg");
	z_odraz_l = nahrajZvuk(PATH_SOUND "odraz-l.ogg");

	zapnuta=1;
#endif
}

void setZvukyStatus(int n)
{
	zapnuta=n;
}

void prehrajZvuk(int n)
{
#ifndef NO_SOUND
	Mix_Chunk *zvuk=NULL;

	if(!zapnuta)return;

	switch(n)
	{
		case ZVUK_SMRT:
			zvuk=z_smrt;
		break;
		case ZVUK_ZBRAN:
			zvuk=z_zbran;
		break;
		case ZVUK_VYSTREL1:
			zvuk=z_zvuk1;
		break;
		case ZVUK_VYSTREL2:
			zvuk=z_zvuk2;
		break;
		case ZVUK_VYSTREL3:
			zvuk=z_zvuk3;
		break;
		case ZVUK_MINA:
			zvuk=z_mina;
		break;
		case ZVUK_BONUS:
			zvuk=z_bonus;
		break;
		case ZVUK_TELEPORT:
			zvuk=z_teleport;
		break;
		case ZVUK_NABYJA_SA:
			zvuk=(rand()%2==0?z_nabyjaSa:z_nabyjaSa2);
		break;
		case ZVUK_OPENING:
			zvuk=z_opening;
		break;
		case ZVUK_LASSER:
			zvuk=z_lasser;
		break;
		case ZVUK_ODRAZ_ZBRAN:
			zvuk=z_odraz_zbran;
		break;
		case ZVUK_ODRAZ_R:
			zvuk=z_odraz_r;
		break;
		case ZVUK_ODRAZ_L:
			zvuk=z_odraz_l;
		break;
	}

	if(zvuk==NULL)
	{
		fprintf(stderr, "Chyba: pokus o prehrani nenacteneho zvuku\n");
		return;
	}

	if (Mix_PlayChannel(-1, zvuk, 0)==-1)
	{
		fprintf(stderr, "Nelze prehrat zvuk : %s\n", Mix_GetError());
		return;
	}
#endif
}

void quitAudio()
{
#ifndef NO_SOUND
	//uvolni zvuky
	Mix_FreeChunk(z_zvuk1);
	Mix_FreeChunk(z_zvuk2);
	Mix_FreeChunk(z_zvuk3);
	Mix_FreeChunk(z_mina);
	Mix_FreeChunk(z_smrt);
	Mix_FreeChunk(z_zbran);
	Mix_FreeChunk(z_bonus);
	Mix_FreeChunk(z_teleport);
	Mix_FreeChunk(z_nabyjaSa);
	Mix_FreeChunk(z_nabyjaSa2);
	Mix_FreeChunk(z_opening);
	Mix_FreeChunk(z_lasser);
	Mix_FreeChunk(z_odraz_zbran);
	Mix_FreeChunk(z_odraz_r);
	Mix_FreeChunk(z_odraz_l);

	//uvolni audio	
	Mix_CloseAudio();
#endif
}
